#include <reg552.h>
#include <intrins.h>
#include <absacc.h>
#include <stdlib.h>
#include <stdio.h>
#include <rtx51tny.h>
#include <dsw.h>

#define	SIMULx

#define REGULATOR	0
#define	ERRORS		1

//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define	XTAL 18.432E6
#define Frekvencia_horna 700							//478
#define Frekvencia_dolna 300
#define Cislo (XTAL/(12*2*Frekvencia_horna))
#define Tzmin ((XTAL*100E-6)/12)						//Tzmin=100us
#define Hranica_Dolna  (XTAL/(12*2*(Frekvencia_dolna)))	// Dolna frekvencia (zaraska dolnej frekvencie)
#define Hranica_Horna  (XTAL/(12*2*(Frekvencia_horna)))
//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define WLE 0x10
#define INTERVAL 200							        //Watchdog cca. 200ms
#define Watchdog ((PCON|=WLE),(T3=0x100-INTERVAL))
#define	DCapture (IEN1&=~0x01)							//Zakazem akceptovanie pereusenia od Capture jednotky, kde je Tz
#define	ECapture (IEN1|=0x01)							//Povolim akceptovanie pereusenia od Capture jednotky, kde je Tz
#define	DCompare //(IEN1&=~0x10)							//Pozastavim pocas vypoctu Perioda++ a Perioda-- generovanie impulzu v Compare jednotke
#define	ECompare //(IEN1|=0x10)							//Povolim Compare jednotku
//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

sbit I1 = P4^0;

int Per = 0x0000, Perioda = Cislo;

void Impuls1(void) interrupt 11 using 2
{
 #define Dlzka 4			//Dlzka = 4 cca.20us s ostatnym kodom. Tu sa meni scasti dlzka impulzu !!!
 unsigned char Temp=Dlzka;
 I1=1;
 Per+=Perioda;
 CMH0=high(Per);
 CML0=low(Per); 			//Generujem pre podmienku CMH0+CML0=TMH2+TML2 
 Watchdog;
 while(--Temp!=0x00);
 I1=0;
 CMI0=0;
}

struct Measure		{int Beg,End,Len;};						//Definujem strukturu pre meranie Tz a Pe
struct Measure Tz = {0x0000,0x0000,Tzmin};			        //Uchovavam Tz
struct Measure Pe = {0x0000,0x0000,(int)(Tzmin*6.3)};		//Uchovavam dlzku periody Pe

void Capture(void) interrupt 6 using 3
{
 #define CTP0 0x01
 #define CTN0 0x02
 switch(CTCON & (CTP0+CTN0))
 {
  case	CTP0	:	Tz.End=Pe.Beg=(256*CTH0+CTL0); Tz.Len=(Tz.End-Tz.Beg); break;
  case	CTN0	:	Tz.Beg=Pe.End=(256*CTH0+CTL0); Pe.Len=(Pe.End-Pe.Beg); break;
 }
 CTCON^=(CTP0+CTN0);
 CTI0=0;
}

void Timer(void) interrupt 3	//Vsetko napisem v assembleri ???
{
 P1^=0x01;
}

void Initialize(void)
{
 CMH0=high(Per);
 CML0=low(Per);
 IEN1|=0x10;			//Povolim komparacnu jednotku
 IEN1|=0x01;			//Povolujem Capture jednotku
 IP1|=0x10;				//Komparacna jednotka s vacsou prioritou ...
 TM2CON=0x01;			//T2 s frekvenciou fosc/1 pretecie za 1*40.27ms
 CTCON|=CTN0;			//Citlivost na prechod "0" CAPTURE 1
}

sbit ZAVES_SA     = P3^5;													//Od tohoto okamihu znizujem 
sbit SOM_ZAVESENY = P3^4; 

//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define Kp 2.800														//Proporcionalna konstanta Kp
#define dT 0.005														//Interval vzorkovania pe regulator dT=20ms
#define Ti 0.041														//Integracna konstanta Ti, je potrebne ju prehodnotit pri zmene menica ...
#define Td 0.001														//Derivacna konstanta Td, ovplyvnuje strmost narastu pri zmene reg. odchylky, vsima si totiz strmost narastu odchylky !!!
//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
float E0 = 0.000;									        			//Regulacna odchylka E0 nova
float E1 = 0.000;							        					//Regulacna odchylka E1 stara
float E2 = 0.000;							        					//Regulacna odchylka E2 moc stara
float Y0 = 0.000;							        			    	//Akcna velicina Y0	nova pre regulator
float Y1 = (float)Hranica_Horna;        		    					//Akcna velicina Y1 stara pre regulator
float KA = (Kp*(1.0+dT/(2.0*Ti)+(Td/dT)));    					    	//Konstanta KA prirastkoveho regulatora
float KB = (Kp*(1.0-dT/(2.0*Ti)+(2.0*Td/dT)));    					    //Konstanta KB prirastkoveho regulatora
float KC = (Kp*(Td/dT));					    					    //Konstanta KC prirastkoveho regulatora
//------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#define Pomer 4.500														    //4.9 bolo OK ???

void Regulator(void) _task_ REGULATOR
{
 unsigned char Counter0,Counter1;
 float Pomer_vert; 														//Nastavenie Pomer_vert pri simulacii signalov programu START
 bit R=0;
 Initialize();
 #ifdef SIMUL
 ET1=1;
 TR1=1;
 #endif
 while(1)
  {
   os_wait(K_TMO,5,NULL);												//Cakam pokial nepride povel na uplynutie casu
   if(ZAVES_SA==1)
    {
     Watchdog;
	 DCapture;															//Zakazem na chvilu prerusenie od CAPTURE
	 Pomer_vert=(float)Pe.Len/(float)Tz.Len;							//Vypocet trva cca. 800us  pri 18.432MHz 
	 ECapture;
	 E0=Pomer-Pomer_vert;		 		   		                        //Vypocet trva cca. 131us  pri 18.432MHz 
     Y0=Y1+(KA*E0)-(KB*E1)+(KC*E2);                                    	//Vypocet trva cca. 1064us pri 18.432MHz
	 if(Y0>=Hranica_Dolna) Y0=Hranica_Dolna;
	 if(Y0<=Hranica_Horna) Y0=Hranica_Horna;
	 DCompare; 
	 Perioda=(int)Y0;
	 ECompare;
     Y1=Y0;
	 E2=E1;
     E1=E0;
	 if(Pomer_vert>Pomer)       R=1;     					 
	 if(Pomer_vert<(Pomer-0.9)) R=0; 
     if(R==1) {Counter0++; Counter1=0x00;}
	 if(R==0) {Counter1++; Counter0=0x00;}
	 if(Counter0>=5) {SOM_ZAVESENY=1; Counter0=0x00; os_wait(K_TMO,20,NULL);}
	 if(Counter1>=20){SOM_ZAVESENY=0; Counter1=0x00; os_wait(K_TMO,20,NULL);}
    }
   else
	{
     Watchdog;
	 os_wait(K_TMO,200,NULL);
	 Y0=Y1=(float)Cislo;
	 Perioda=Cislo;
	 SOM_ZAVESENY=0;
	 Counter0=0;
	 Counter1=0;						   
	}
  }
}